@@ -4,10 +4,6 @@
 
 #define QT_FEATURE_linux_netlink -1
 
-#define QT_FEATURE_openssl 1
-
-#define QT_FEATURE_openssl_linked 1
-
 #define QT_FEATURE_system_proxies 1
 
 #define QT_FEATURE_networklistmanager -1
